/*
 *  AudioManager.cpp
 *  AudioDiary
 *
 *  Created by Roy Fejgin on 3/8/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "AudioManager.h"
#include "mo_audio.h"
#include "Clip.h"

#define DEFAULT_SAMPLE_RATE

#define MAX_AUDIO_WINDOW    1.01       // in minutes
#define DEF_AUDIO_WINDOW    1.0    // in minutes
#define FILE_WRITE_THRESH   0.9    // as a fraction of the circular buffer maximum capacity 
#define SEC_PER_MIN         60

AudioManager::AudioManager()
{
    int sr = MoAudio::m_hwSampleRate;
    int maxLookback = sr*MAX_AUDIO_WINDOW*SEC_PER_MIN;
    m_delaySamples = sr*DEF_AUDIO_WINDOW*SEC_PER_MIN;
    m_bufIndex = 0;
    m_isRecording = false;
    m_samplesSinceSave = 0;
    
    for (int i=0; i<NUM_BUFFERS; i++) {
        m_buffers[i] = new CircularBuffer<SAMPLE>(maxLookback+1,
                                                  m_delaySamples,
                                                  true,
                                                  0);
    }
    m_recorder = new AdRecorder();
    m_analyzer = new TapAnalyzer(*m_buffers[m_bufIndex], maxLookback );
    
    const char *startClip = [[[NSBundle mainBundle]
                              pathForResource:@"4beep"
                              ofType:@"wav"] UTF8String];        
    
    m_clipPlayer = new AudioClipPlayer(startClip, 0.3);
    
}


SAMPLE AudioManager::tick(SAMPLE input)
{
    SAMPLE output = 0.0;
    // Save input in circular buffer
    m_buffers[m_bufIndex]->pushPop(input);
    
    // Look for tap
    TapType tap = m_analyzer->tick(input);
    
    // Get audio clip player output
    output+= m_clipPlayer->tick();
    
    m_samplesSinceSave++;
    
    int bufIndex = m_bufIndex;
    
    NSString* file;
    CLLocationCoordinate2D coord;
    
    
    // Tap detected?
    switch (tap) {
        case TAP_START:
            // TODO: locking?
            
            // start recording
            m_recorder->startRecord();
            m_isRecording = true;
            
            // Save start time
            m_startTime = [NSDate timeIntervalSinceReferenceDate];
            
            // switch to new buffer
            advanceBuffer();
            
            // save old buffer
            m_recorder->writeBuffer(m_buffers[bufIndex], MIN(m_delaySamples, m_samplesSinceSave));
            m_samplesSinceSave = 0;
            // Play record indication
            m_clipPlayer->playClip();
            
            
            // Notify UI about update record status change
            [[NSNotificationCenter defaultCenter] 
             postNotificationName:@"RecordStatus"
             object:[NSNumber numberWithBool:YES]];
            
            break;
        case TAP_STOP:
        {
            // save to file
            m_recorder->writeBuffer(m_buffers[bufIndex], m_samplesSinceSave,true);
            m_samplesSinceSave = 0;
            // stop recording
            file = m_recorder->stopRecord();
            m_isRecording = false;
            // Sound stop indication
            m_clipPlayer->playClip();
            
            // Calculate length
            NSDate* date = [[[NSDate alloc ]initWithTimeIntervalSinceReferenceDate: m_startTime] autorelease];
            NSDate* dateNow = [NSDate date];
            NSTimeInterval interval = [dateNow timeIntervalSinceDate:date];
            
            // Create clip object
            Clip clip(coord, m_startTime, interval, file );
            m_clipManager->addClip(&clip);
            
            // Notify views about update record status change
            [[NSNotificationCenter defaultCenter] 
             postNotificationName:@"RecordStatus"
             object:[NSNumber numberWithBool:NO]];            
            
            // Add new clip to database
            coord.longitude = 0;
            coord.latitude = 0;
            
            
        }
            break;
            
        case TAP_NONE:
            break;
            
        default:
            // not yet handeled
            assert(0);
            break;
    }
    
    float capacity = m_buffers[0]->getMaxCapacity();
    if (m_isRecording && (m_samplesSinceSave > (capacity *.9))) {
        bufIndex = m_bufIndex;
        advanceBuffer();
        m_recorder->writeBuffer(m_buffers[m_bufIndex], m_samplesSinceSave);
        m_samplesSinceSave = 0;
    }
    
    return output;
}


