/*
 *  HarmonyModel.cpp
 *  
 *
 *  Created by Roy Fejgin on 2/13/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "HarmonyModel.h"
#import "my_util.h"
#import <algorithm>

#define ACTIVITY_THRESH    0.0001
#define LEAK_FACTOR        0.9

HarmonyModel:: HarmonyModel(int frameSize, float sampleRate)
{
    m_chordSelector = new ChordSelector(1, 1.5);

    m_harmonyIndex = 0;
    for (int i=0; i<HE_LAST; i++) {
        m_harmonyAudio[i] = new HarmonyAudio();
        m_adsr[i] = new MyADSR((int)(sampleRate/2), 1, 1.0, (int)(sampleRate/2), false);
    }
    
    m_pitchDetector = new PitchDetector(frameSize, sampleRate);
    m_onePole = new OnePole(0.99);
    
    initFreqMapping();
    
    m_currentNote = 0;
    m_currentFreq = 0;
    m_targetFreq = 0;
    m_pitchValid = false;
}


SAMPLE HarmonyModel::tick(SAMPLE input)
{
    SAMPLE output = 0;
    bool newPitch = false;
    
    SAMPLE energy = m_onePole->tick(input*input);
    newPitch = m_pitchDetector->tick(input);
    
    
    for (int i=0; i<HE_LAST; i++) {
        if (!m_adsr[i]->isNoteDone()) {
            output+= m_adsr[i]->tick(input)*m_harmonyAudio[i]->tick(input);
        }
    }
    
    // scale down to avoid clipping
    output /= HE_LAST;

    if(newPitch && energy > ACTIVITY_THRESH) {
        int note;
        m_pitchValid = true;
        m_targetFreq = m_pitchDetector->getPitch();
        float noteFreq;
        note = lookupFreq(m_currentFreq, &noteFreq)%12;
        
        for (int i=0; i<HE_LAST; i++) {
            m_harmonyAudio[i]->setFundFreq(m_currentFreq);//m_freqs[note+12]);
        }
        m_currentNote = note;
        
    }
    else {
        m_pitchValid = false;
    }
    
    m_currentFreq = leakyIntegrate(m_targetFreq, m_currentFreq, LEAK_FACTOR);

    
    if (m_harmonyAudio[0]->isFluteOn()) {
        // Make gain dependent on level of input
        // only needed for the STK instrument and not for pitch shifting
        // since pitch shifting is generated from the input signal and will
        // therefore already have the right gain.
        output *= sqrt(energy);
    }

    return output;
    
}

void HarmonyModel::touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * view)
{
    bool changed = false;
    ChordType chordType;
    
    changed = m_chordSelector->touchEvent(touchPts, view, &chordType);
    
    if (changed) {
        vector<float> triadRatios;
        vector<float> extRatios;
        buildTriad(chordType, triadRatios);
        buildExt(chordType, extRatios);
        
        //MUTEX_LOCK(&m_mutex);
        
        if (chordType.triad == CH_NONE) {
            m_adsr[0]->noteOff();
        }
        else {
            m_harmonyAudio[0]->setHarmony(m_freqs[m_currentNote+12], triadRatios);
            m_adsr[0]->noteOn();
        }

        if (chordType.ext == EXT_NONE) {
            m_adsr[1]->noteOff();
        }
        else {
            m_harmonyAudio[1]->setHarmony(m_freqs[m_currentNote+12], extRatios);
            m_adsr[1]->noteOn();
        }

        //MUTEX_UNLOCK(&m_mutex);
        
    }
}

void HarmonyModel::buildTriad(ChordType cType, vector<float>& ratios)
{
    switch (cType.triad) {
        case CH_MAJ:
            ratios.push_back(degToRatio(0));
            ratios.push_back(degToRatio(4));
            ratios.push_back(degToRatio(7));
            break;
        case CH_MIN:
            ratios.push_back(degToRatio(0));
            ratios.push_back(degToRatio(3));
            ratios.push_back(degToRatio(7));
            break;
        case CH_DIM:
            ratios.push_back(degToRatio(0));
            ratios.push_back(degToRatio(3));
            ratios.push_back(degToRatio(6));
            break;
        case CH_AUG:
            ratios.push_back(degToRatio(0));
            ratios.push_back(degToRatio(4));
            ratios.push_back(degToRatio(8));
            break;
        case CH_NONE:
            break;
        default:
            assert(0);
            break;
    }
    
}

void HarmonyModel::buildExt(ChordType cType, vector<float>& ratios)
{
    switch (cType.ext) {
        case EXT_SEV_MAJ:
            ratios.push_back(degToRatio(11));
            break;
        case EXT_SEV_MIN:
            ratios.push_back(degToRatio(10));
            break;
        case EXT_SEV_DIM:
            ratios.push_back(degToRatio(9));
            break;
        case EXT_NONE:
            break;
    }
}

void HarmonyModel::initFreqMapping()
{
    for (int i=0; i<NOTE_RANGE; i++) {
        m_freqs[i] = A2Freq*degToRatio(i);
    }
}

int HarmonyModel::lookupFreq(float freq, float* outFreq)
{
    int result;
    // use binary search to find closets pair of frequencies
    pair<float*, float*> where = equal_range(m_freqs, m_freqs+NOTE_RANGE, freq);
    
    if((freq - *where.first) > (*where.second - freq)) {
        result = where.second - m_freqs;
        *outFreq = *where.second;
    }
    else {
        result = where.first - m_freqs;
        *outFreq = *where.first;
    }
    
    return result;
}
