/*
 *  PitchDetector.cpp
 *  Uses autocorrelation to detect pitch
 *
 *  Created by Roy Fejgin on 2/15/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#import "PitchDetector.h"
#import "my_util.h"



PitchDetector::PitchDetector(int frameSize, float sampleRate) :
    m_frameSize(2*frameSize*zeroPadFactor), m_sampleRate(sampleRate), m_index(0), m_pitch(0)
{
    // Use large FFT size for frequency accuracy. Also leave room for zero padding
    m_buffer = new SAMPLE[m_frameSize];
    memset(m_buffer, 0, m_frameSize*sizeof(SAMPLE));
    
    m_autoCorrBuffer = new SAMPLE[m_frameSize];
    memset(m_autoCorrBuffer, 0, m_frameSize*sizeof(SAMPLE));

    m_window = new SAMPLE[(int)(m_frameSize/zeroPadFactor)];
    
    // Initialize hanning window
    hanning(m_window, m_frameSize/zeroPadFactor);
}


bool PitchDetector::tick(SAMPLE input)
{
    bool changed = false;
    
    // Store sample
    m_buffer[m_index] = input;
    m_index++;
        
    if (m_index >= (m_frameSize/zeroPadFactor)) {
        m_index = 0;
        
        // window
        apply_window(m_buffer, m_window, m_frameSize/zeroPadFactor);
        
        // fft
        rfft(m_buffer, m_frameSize/2, FFT_FORWARD);
        
        // compute auto correlation
        computeAutoCorrelation((complex*)m_buffer, (complex*)m_autoCorrBuffer, m_frameSize/(sizeof(complex)/sizeof(SAMPLE)));

        // inverse fft
        rfft(m_autoCorrBuffer, m_frameSize/2, FFT_INVERSE);

        // find first peak of autocorrleation
        float sampleNum = findFirstPeak(m_autoCorrBuffer, m_frameSize);
        
        if (sampleNum) {
            float pitch = m_sampleRate / sampleNum;
            
            if (pitch > 100 && pitch <3000) {
                changed = true;
                m_pitch = pitch;
            }
            DEBUG_LOG(@"Pitch = %f Hz", pitch);
            DEBUG_LOG(@"Saved Pitch = %f Hz", m_pitch);
        }
    }
    
    return changed;
}


void PitchDetector::computeAutoCorrelation(complex* inFftBuffer, complex* outBuffer, int length)
{
    
    // make copies
    memcpy( outBuffer, inFftBuffer, sizeof(complex)*length );

    
    for(int i=0; i<length; i++){
        // conjugate first copy
        inFftBuffer[i].im *= -1;
        
        // complex multiply
        outBuffer[i] = complexMultiply(&inFftBuffer[i], &outBuffer[i]);
    }
}